var Ankaa = Ankaa || {};
Ankaa.Client = Ankaa.Client || {};
Ankaa.Client.JSEngine = Ankaa.Client.JSEngine || {};

Ankaa.Client.JSEngine.Kickstart = function(win)
{
    var self = this;

    var isVuplexAvailable = null;
    
    var init = function()
    {
        //console.log('Ankaa.Client.JSEngine.Kickstart enter init');
        //console.log('Ankaa.Client.JSEngine.Kickstart url is: ' + document.location.href);
        //console.log('Ankaa.Client.JSEngine.Kickstart userAgent is: ' + navigator.userAgent);
        // we are here because onDomLoaded has been triggered 
        // if we are in VR, the url contains ?waitVuplex=1 and we must wait that window.vuplex becomes available
        // on real PC/Mac webbrowser we don't need vuplex because we simulate VR communication using GameSimulatorToolkit
        initDelayed(document.location.href.indexOf('waitVuplex=1') >-1 || navigator.userAgent.indexOf('AnkaaBrowser') > -1);
    }

    var initDelayed = function(waitForVuplex){
        try
        {
            var initialized = false;
            if(!waitForVuplex)
            {
                initializeKickstart();
                try{
                    host.messageFromHost.connect(
                        (sender, args) => {
                            //console.log('Ankaa.Client.JSEngine.Kickstart enter message from host event listener'); 
                            onMessageReceived.call(self, JSON.parse(args.Message));
                        }
                    );
                }
                catch(e1)
                {
                    console.log("Cannot connect to JSEngine environment host");
                }
                self.sendMessage('ready', '', '');
                initialized = true;
            }
            else
            {
                isVuplexAvailable = typeof(globalThis.vuplex) != 'undefined';
                if(isVuplexAvailable)
                {
                    initializeKickstart();
                    globalThis.vuplex.addEventListener('message', function(evt){ 
                        //console.log('Ankaa.Client.JSEngine.Kickstart enter message from host event listener'); 
                        onMessageReceived.call(self, JSON.parse(evt.data)); 
                    })
                    self.sendMessage('ready', '', '');
                    initialized = true;
                }
                else
                {
                    setTimeout(function(){ initDelayed(waitForVuplex) }, 200); // retry in 200ms
                }
            }
            
                
    
            //console.log('Ankaa.Client.JSEngine.Kickstart loaded');
        }
        catch(e1)
        {
            console.log('Ankaa.Client.JSEngine.Kickstart exception: '+e1.message+'\n'+e1.stack);
        }
        
    }

    var onMessageReceived = function(msg)
    {
        try
        {
            //console.log(JSON.parse(msg));
            if(msg['category'] == 'dummy-js-code'/* && msg['action'] == 'pong'*/)
            {
                console.log('JS side received a pong from C#');
                document.body.innerHTML += '<br/>' + JSON.stringify(msg);
            }
            else if(msg["category"] == "debug-console" && msg["action"] == "execute-instruction")
            {
                //console.log("JSEngineScreen received a JS command: " + msg["parameters"][0]);
                console.log(">>> " + msg["parameters"][0]);
                eval(msg["parameters"][0]);
            }
            else
            {
                globalThis.$A.processMessage(msg);
            }
        }
        catch(e1)
        {
            console.log('Ankaa.Client.JSEngine.Kickstart JS exception in onMessageReceived method: '+e1.message+'\n'+e1.stack);
        }

        
    }
    
    var sendRawMessage = function(json)
    {
        // if vuplex exists
        if(isVuplexAvailable)
        {
            globalThis.vuplex.postMessage(json);
        }
        else if(navigator.userAgent.indexOf('AnkaaHeadless')>-1)
        {
            host.messageToHost(json);
        }
        else
        {
            console.log('should send message to c#:', json);
        }
    }

    self.sendMessage = function(category, action, parameters, id = "-", responseFor = "-")
    {
        sendRawMessage(JSON.stringify({'category': category, 'action': action, 'parameters': parameters, 'id': id, 'responseFor': responseFor}));
    }
    
    var initializeKickstart = function()
    {
        document.body.innerHTML = '<u>Ankaa Interpreter JS Engine Bridge</u>';
        /*document.body.style.backgroundColor = "#EFEFEF";*/
        
        globalThis.$A = new Ankaa.Client.JSEngine.SceneAPI(
            (function(){ return function(category, action, parameters, id, reponseFor){ self.sendMessage.call(self, category, action, parameters, id, reponseFor); }; })()
        );
        
        
        // below code is to test async support
        /*(async function() { 
            await $delay(2000);
            console.log('send message to host');
            self.sendMessage('js-engine-screen', 'ping', '');
        })();*/
    }

    init.call(self);
}

var $delay = async function(durationMs)
{
    if(navigator.userAgent.indexOf('AnkaaHeadless')>-1)
        await Task.Delay(durationMs); 
    else
        await new Promise(r => setTimeout(r, durationMs));
}


if(document.readyState == 'loading')
{
    window.addEventListener('DOMContentLoaded', (event) => { 
        if(typeof(globalThis.__Ankaa_Client_JSEngine_Kickstart) == 'undefined'){
            globalThis.__Ankaa_Client_JSEngine_Kickstart = new Ankaa.Client.JSEngine.Kickstart(globalThis); 
        }
    });
}
else
{
    if(typeof(globalThis.__Ankaa_Client_JSEngine_Kickstart) == 'undefined'){
        globalThis.__Ankaa_Client_JSEngine_Kickstart = new Ankaa.Client.JSEngine.Kickstart(globalThis);
    }
}
