var Ankaa = Ankaa || {};
Ankaa.Client = Ankaa.Client || {};
Ankaa.Client.JSEngine = Ankaa.Client.JSEngine || {};

Ankaa.Client.JSEngine.StatesCache = function()
{
    var self = this;
    
    var cachedGameObjects = {};

    var init = function()
    {
        //console.log("States_cache initialized");
    }

    self.add = function(goName, go)
    {
        cachedGameObjects[goName] = go;
    }

    /**
     * 
     * @param {String} goName 
     * @returns {Ankaa.Client.JSEngine.GameObject}
     */
    self.get = function(goName)
    {
        if(!cachedGameObjects.hasOwnProperty(goName))
            return null;
        return cachedGameObjects[goName];
    }

    self.remove = function(goName)
    {
        if(cachedGameObjects.hasOwnProperty(goName))
            delete cachedGameObjects[goName];
    }
    
    self.clearAll = function()
    {
        cachedGameObjects = {};
    }

    init.call(self);
}

Ankaa.Client.JSEngine.Event = function(name, args)
{
    var self = this;

    self.name = name;
    self.args = args;

    var init = function()
    {

    }

    init.call(self);
}

Ankaa.Client.JSEngine.EventTarget = function()
{
    var self = this;

    var callbacks = {};

    var init = function()
    {

    }

    /**
     * 
     * @param {Ankaa.Client.JSEngine.Event} evt 
     */
    self.dispatchEvent = function(evt)
    {
        if(callbacks.hasOwnProperty(evt.name))
        {
            for(var idx in callbacks[evt.name])
            {
                callbacks[evt.name][idx](evt.args);
            }
        }
    }

    self.addEventListener = function(name, callback)
    {
        if(!callbacks.hasOwnProperty(name))
        {
            callbacks[name] = [];
        }
        callbacks[name].push(callback);
    }

    init.call(self);
}


/**
 * 
 * @param {Ankaa.Client.JSEngine.BridgeClient} jsBridgeInstance 
 */
Ankaa.Client.JSEngine.GameObject = function(jsBridgeInstance)
{
    var self = this;
    
    self.transform = null;

    self.name = "";

    var init = function()
    {    
        Ankaa.Client.JSEngine.EventTarget.call(self); // call parent constructor
        self.transform = new Ankaa.Client.JSEngine.Transform();
    }

    this.audioClipPlay = function()
    {
        jsBridgeInstance.callGameObjectMethod("audioclip-play", [self.name]);
    }

    this.audioClipStop = function()
    {
        jsBridgeInstance.callGameObjectMethod("audioclip-stop", [self.name]);
    }

    /**
     * 
     * @param {Ankaa.Client.JSEngine.Vector3} newPosition 
     */
    this.setBodyPosition = function(newPosition)
    {
        jsBridgeInstance.callGameObjectMethod("set-body-position", [self.name, newPosition.toSerialized()]);
    }
    
    /**
     * 
     * @param {Ankaa.Client.JSEngine.Quaternion} newRotationQuaternion 
     */
    this.setBodyRotationFromQuaternion = function(newRotationQuaternion)
    {
        jsBridgeInstance.callGameObjectMethod("set-body-rotation-quat", [self.name, newRotationQuaternion.toSerialized()]);
    }

    /**
     * 
     * @param {Ankaa.Client.JSEngine.Vector3} newRotationDegres 
     */
    this.setBodyRotationFromDegres = function(newRotationDegres)
    {
        jsBridgeInstance.callGameObjectMethod("set-body-rotation-degres", [self.name, newRotationDegres.toSerialized()]);
    }

    /**
     * 
     * @param {Akaa.Client.JSEngine.Vector3} impulseToAdd 
     */
    this.addTorqueImpulse = function(impulseToAdd)
    {
        jsBridgeInstance.callGameObjectMethod("add-body-torque-impulse", [self.name, impulseToAdd.toSerialized()]);
    }
    
    /**
     * 
     * @param {Akaa.Client.JSEngine.Vector3} impulseToAdd 
     */
    this.addImpulse = function(impulseToAdd)
    {
        jsBridgeInstance.callGameObjectMethod("add-body-impulse", [self.name, impulseToAdd.toSerialized()]);
    }

    /**
     * 
     * @param {Akaa.Client.JSEngine.Vector3} loadUrl 
     */
     this.loadUrl = function(urlToLoad)
     {
         jsBridgeInstance.callGameObjectMethod("load-url", [self.name, urlToLoad]);
     }

    init.call(self);
}
Ankaa.Client.JSEngine.GameObject.prototype = Object.create(Ankaa.Client.JSEngine.EventTarget.prototype);



Ankaa.Client.JSEngine.Transform = function()
{
    var self = this;
    
    self.position = null;
    self.rotation = null;

    var init = function()
    {
        self.position = new Ankaa.Client.JSEngine.Vector2(0, 0);
        self.rotation = new Ankaa.Client.JSEngine.Quaternion(0, 0, 1, 0);
    }

    init.call(self);
}

Ankaa.Client.JSEngine.Vector2 = function(x, y)
{
    var self = this;

    self.x = 0;
    self.y = 0;

    var init = function()
    {
        self.x = x;
        self.y = y;
    }

    init.call(self);
}

Ankaa.Client.JSEngine.Vector3 = function(x, y, z)
{
    var self = this;

    self.x = 0;
    self.y = 0;
    self.z = 0;

    var init = function()
    {
        self.x = x;
        self.y = y;
        self.z = z;
    }

    self.toSerialized = function()
    {
        return self.x.toString() + ";" + self.y.toString() + ";" + self.z.toString();
    }

    init.call(self);
}

Ankaa.Client.JSEngine.Quaternion = function(w, x, y, z)
{
    var self = this;

    self.w = 0;
    self.x = 0;
    self.y = 0;
    self.z = 0;

    var init = function()
    {
        self.w = w;
        self.x = x;
        self.y = y;
        self.z = z;
    }

    init.call(self);
}